@@ -22,7 +22,6 @@ import os
 from libqtopensesame.misc.config import cfg
 from libqtopensesame.extensions import BaseExtension
 from libopensesame.oslogging import oslogger
-from jupyter_tabwidget import ConsoleTabWidget
 from qtpy.QtWidgets import QDockWidget, QShortcut
 from qtpy.QtCore import Qt
 from qtpy.QtGui import QKeySequence
@@ -31,9 +30,15 @@ _ = translation_context(u'JupyterConsole', category=u'extension')
 
 
 class JupyterConsole(BaseExtension):
+    
+    preferences_ui = 'extensions.JupyterConsole.preferences'
 
+    @BaseExtension.as_thread(wait=2000)
     def event_startup(self):
 
+        from jupyter_tabwidget import ConsoleTabWidget
+
+        self.set_busy(True)
         self._jupyter_console = ConsoleTabWidget(self.main_window)
         self._dock_widget = QDockWidget(u'Console', self.main_window)
         self._dock_widget.setObjectName(u'JupyterConsole')
@@ -50,9 +55,21 @@ class JupyterConsole(BaseExtension):
             self._focus,
             context=Qt.ApplicationShortcut
         )
+        self.set_busy(False)
+        
+    def fire(self, event, **kwdict):
+        
+        if event != 'startup':
+            oslogger.debug('ignoring events until after startup')
+            return
+        JupyterConsole.fire = BaseExtension.fire
+        self.fire(event, **kwdict)
 
     def activate(self):
 
+        if not hasattr(self, '_jupyter_console'):
+            oslogger.debug('ignoring activate until after startup')
+            return
         self._set_visible(not cfg.jupyter_visible)
 
     def event_run_experiment(self, fullscreen):
@@ -70,13 +87,21 @@ class JupyterConsole(BaseExtension):
 
         self._jupyter_console.add(kernel=kernel)
 
-    def event_jupyter_run_file(self, path):
+    def event_jupyter_run_file(self, path, debug=False):
 
         self._set_visible(True)
         if not os.path.isfile(path):
             return
         self._jupyter_console.current.change_dir(os.path.dirname(path))
-        self._jupyter_console.current.run_file(path)
+        if debug:
+            self._jupyter_console.current.run_debug(
+                path,
+                breakpoints=self.extension_manager.provide(
+                    'pyqode_breakpoints'
+                )
+            )
+        else:
+            self._jupyter_console.current.run_file(path)
 
     def event_jupyter_change_dir(self, path):
 
@@ -116,22 +141,61 @@ class JupyterConsole(BaseExtension):
 
     def provide_jupyter_workspace_name(self):
 
-        return self._jupyter_console.current.name
+        try:
+            return self._jupyter_console.current.name
+        except AttributeError:
+            return None
+        
+    def provide_workspace_kernel(self):
+        
+        try:
+            return self._jupyter_console.current._kernel
+        except AttributeError:
+            return None
 
     def provide_jupyter_workspace_globals(self):
 
         return self.get_workspace_globals()
 
+    def provide_jupyter_list_workspace_globals(self):
+
+        return self.list_workspace_globals()
+
     def provide_jupyter_workspace_variable(self, name):
 
         return self._jupyter_console.current.get_workspace_variable(name)
+        
+    def provide_jupyter_kernel_running(self):
+        
+        try:
+            return self._jupyter_console.current.running()
+        except AttributeError:
+            return False
+
+    def provide_jupyter_check_syntax(self, code):
+
+        return self._jupyter_console.current.check_syntax(code)
 
     def get_workspace_globals(self):
 
-        return self._jupyter_console.current.get_workspace_globals()
+        try:
+            return self._jupyter_console.current.get_workspace_globals()
+        except AttributeError:
+            return {u'no reply': None}
+
+    def list_workspace_globals(self):
+
+        try:
+            return self._jupyter_console.current.list_workspace_globals()
+        except Exception as e:
+            print(e)
+            return []
 
     def event_close(self):
 
+        if not hasattr(self, '_jupyter_console'):
+            oslogger.debug('ignoring close all')
+            return
         self._jupyter_console.close_all()
 
     def _set_visible(self, visible):
